#include "SyntheseVocale.h"
#include "espeak/speak_lib.h"

// Constructeurs et Destructeur
SyntheseVocale::SyntheseVocale() : texte(""), voice("mb-fr1"), speed(130), volume(80), pitch(40), range(60), punctuation(0), gender(2), eSpeakVersionInfo(NULL), samplerate(22050)
{
   version();
      
   const char *data_path = NULL; // use default path for espeak-data
   samplerate = espeak_Initialize(AUDIO_OUTPUT_PLAYBACK, 0, data_path, 0);
   //samplerate = espeak_Initialize(AUDIO_OUTPUT_SYNCH_PLAYBACK, 0, data_path, 0);
   cout << "Samplerate : " << samplerate << endl;
   
   //liste(); // liste toutes les langues disponibles
   // Attention : les langues MBROLA ne sont pas affichées mais potentiellement disponibles aussi
      
   setVoice(voice);  
   
   // 0=no punctuation
   // 2=female   
   reglages();
}

SyntheseVocale::SyntheseVocale(int speed, int volume, int pitch, int range) : texte(""), voice("fr"), speed(speed), volume(volume), pitch(pitch), range(range), punctuation(0), gender(2), eSpeakVersionInfo(NULL), samplerate(22050)
{
   version();
      
   const char *data_path = NULL; // use default path for espeak-data
   samplerate = espeak_Initialize(AUDIO_OUTPUT_PLAYBACK, 0, data_path, 0);
   //samplerate = espeak_Initialize(AUDIO_OUTPUT_SYNCH_PLAYBACK, 0, data_path, 0);
   //cout << "Samplerate : " << samplerate << endl;
   
   //liste(); // liste toutes les langues disponibles
   // Attention : les langues MBROLA ne sont pas affichées mais potentiellement disponibles aussi
      
   setVoice(voice);
   
   // 0=no punctuation
   // 2=female   
   reglages();   
}

SyntheseVocale::~SyntheseVocale()
{
   espeak_Terminate();
}

// Accesseurs
string SyntheseVocale::getVoice() const
{
   return voice;
}

int  SyntheseVocale::getSpeed() const
{
   return speed;
}

int  SyntheseVocale::getVolume() const
{
   return volume;
}

int  SyntheseVocale::getPitch() const
{
   return pitch;
}

int  SyntheseVocale::getRange() const
{
   return range;
}

int  SyntheseVocale::getPunctuation() const
{
   return punctuation;
}

int  SyntheseVocale::getGender() const
{
   return gender;
}

void SyntheseVocale::setVoice(string voice)
{
   if(espeak_SetVoiceByName(voice.c_str()) != EE_OK)
         cerr << "espeak_SetVoiceByName error !" << endl;
   else
   {
     cout << "Selected language : " << voice << endl;
     this->voice = voice;
   }   
}

void SyntheseVocale::setSpeed(int speed)
{
   // speed in words per minute Values 80 to 450.
   if(speed >= 80 && speed <= 450)
   {
      this->speed = speed;
      espeak_SetParameter(espeakRATE, speed, 0);
   }
}

void SyntheseVocale::setVolume(int volume)
{
   // volume in range 0-100 (0=silence)
   if(volume >= 0 && volume <= 100)
   {
      this->volume = volume;
      espeak_SetParameter(espeakVOLUME, volume, 0);
   }
}

void SyntheseVocale::setPitch(int pitch)
{
   // base pitch in range 0-100 (50=normal)
   if(pitch >= 0 && pitch <= 100)
   {
      this->pitch = pitch;
      espeak_SetParameter(espeakPITCH, pitch, 0);
   }
}

void SyntheseVocale::setRange(int range)
{
   // pitch range in range 0-100 (0=monotone, 50=normal)
   if(range >= 0 && range <= 100)
   {
      this->range = range;
      espeak_SetParameter(espeakRANGE, range, 0);
   }
}

void SyntheseVocale::setPunctuation(int punctuation)
{
   // 0 no punctuation, 1 say punctuation
   if(range >= 0 && range <= 1)
   {
      this->punctuation = punctuation;
      espeak_SetParameter(espeakPUNCTUATION, punctuation, 0);
   }
}

void SyntheseVocale::setGender(int gender)
{
   // 0=none, 1=male, 2=female
   if(gender >= 0 && gender <= 2)
   {
      this->gender = gender;
      espeak_VOICE *voice_spec = espeak_GetCurrentVoice();
      //memset(voice_spec, 0, sizeof(espeak_VOICE)); // Zero out the voice first
      //voice_spec->languages = "fr";
      //voice_spec->identifier = "mb-fr1";
      voice_spec->gender = gender;
      //voice_spec->age = 25;
      //voice_spec->variant = 1;
      int status = espeak_SetVoiceByProperties(voice_spec);
      //cout << "Status : " << status << endl;
      voice_spec = espeak_GetCurrentVoice();
      //cout << "Language shortsign : " << voice_spec->languages << endl;
      //cout << "Identifier : " << voice_spec->identifier << endl;
      //cout << "Age : " << (int)voice_spec->age << endl;
      //cout << "Gender : " << (int)voice_spec->gender << endl;
      //cout << "Name : " << voice_spec->name << endl;
      //cout << "Variant : " << (int)voice_spec->variant << endl;
      //cout << endl;
   }
}

// Services
void SyntheseVocale::parle(const string texte)
{
   cout << "-- " << texte << endl;
   espeak_Synth((char *)texte.c_str(), texte.length()+1, 0, POS_CHARACTER, 0, espeakCHARS_AUTO, NULL, NULL);
   espeak_Synchronize(); // appel bloquant
}

void SyntheseVocale::muet()
{
   setVolume(0);
}

void SyntheseVocale::liste() const
{
   const espeak_VOICE **voices = NULL;
   espeak_VOICE voice_select;
   const espeak_VOICE *v;
   
   voices = espeak_ListVoices(NULL);
   
   for(int ix=0; (v = voices[ix]) != NULL; ix++)
   {
      cout << "Language shortsign : " << v->languages << endl;
      cout << "Identifier : " << v->identifier << endl;
      cout << "Age : " << v->age << endl;
      cout << "Gender : " << v->gender << endl;
      cout << "Name : " << v->name << endl;
      cout << "Variant : " << v->variant << endl;
      cout << endl;
   }
}

void SyntheseVocale::version()
{
   eSpeakVersionInfo = espeak_Info(NULL);
   cout << "SyntheseVocale version : " << eSpeakVersionInfo << endl;
}

void SyntheseVocale::reglages()
{
   cout << "Gender : " << gender << endl;
   setGender(gender);
   cout << "Speed : " << speed << endl;
   setSpeed(speed);
   cout << "Volume : " << volume << endl;
   setVolume(volume);
   cout << "Pitch : " << pitch << endl;
   setPitch(pitch);
   cout << "Range : " << range << endl;
   setRange(range);
   cout << "Punctuation : " << punctuation << endl;
   setPunctuation(punctuation);
}

